# Copyright 2021-2022 Boris Shminke
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Utils
======

A collection of different useful functions.
"""
import asyncio
import re
from typing import Optional, Tuple

from isabelle_client.compatibility_helper import async_run
from isabelle_client.isabelle__client import IsabelleClient


def get_isabelle_client(server_info: str) -> IsabelleClient:
    """
    get an instance of ``IsabelleClient`` from server info

    >>> server_inf = 'server "test" = 127.0.0.1:10000 (password "pass")'
    >>> print(get_isabelle_client(server_inf).port)
    10000
    >>> get_isabelle_client("wrong")
    Traceback (most recent call last):
        ...
    ValueError: Unexpected server info: wrong

    :param server_info: a line returned by a server on start
    :returns: an Isabelle client
    """
    match = re.compile(
        r"server \".*\" = (.*):(.*) \(password \"(.*)\"\)"
    ).match(server_info)
    if match is None:
        raise ValueError(f"Unexpected server info: {server_info}")
    address, port, password = match.groups()
    isabelle_client = IsabelleClient(address, int(port), password)
    return isabelle_client


def start_isabelle_server(
    log_file: Optional[str] = None,
    name: Optional[str] = None,
    port: Optional[int] = None,
) -> Tuple[str, asyncio.subprocess.Process]:
    """
    start Isabelle server

    >>> import os
    >>> os.environ["PATH"] = "isabelle_client/resources:$PATH"
    >>> print(start_isabelle_server()[0])
    server "isabelle" = 127.0.0.1:9999 (password "test_password")
    <BLANKLINE>

    :param log_file: a log file for exceptional output of internal server and
        session operations
    :param name: explicit server name (default: isabelle)
    :param port: explicit server port
    :returns: a line of server info and server process
    """
    args = (
        "server"
        + (f" -L {log_file}" if log_file is not None else "")
        + (f" -p {str(port)}" if port is not None else "")
        + (f" -n {name}" if name is not None else "")
    )

    async def async_call():
        isabelle_server = await asyncio.create_subprocess_exec(
            "isabelle", *(args.split(" ")), stdout=asyncio.subprocess.PIPE
        )
        return (await isabelle_server.stdout.readline()).decode(
            "utf-8"
        ), isabelle_server

    return async_run(async_call())



# Copyright 2021-2022 Boris Shminke
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Compatibility helper
=====================

A module helping to run
`asyncio <https://docs.python.org/3/library/asyncio.html>`__ in Python 3.6
"""
import asyncio
import sys
from typing import Any, Coroutine


def async_run(a_coroutine: Coroutine) -> Any:
    """
    a simple function which models the behaviour of `asyncio.run` method for
    Python 3.6

    :param a_coroutine: some coroutine to await for
    :returns: whatever the coroutine returns
    """
    if sys.version_info.major == 3 and sys.version_info.minor >= 7:
        # pylint: disable=no-member
        result = asyncio.run(a_coroutine)  # type: ignore
    else:
        result = asyncio.get_event_loop().run_until_complete(a_coroutine)
    return result
